@@ -102,7 +102,7 @@ use constant {
     havePack64   => ( eval { pack('Q>', 1153202979583557643) eq "\x10\x01\0\0\0\0\0\x0B" } ? 1 : 0 ),
 };
 
-our $VERSION = '1.505';
+our $VERSION = '1.603_02';
 our @EXPORT_OK = qw( as_string );
 
 sub as_string {
@@ -310,7 +310,12 @@ sub _counted_header {
 
 sub _neg_integer {
     my($count) = @_;
-   	return pack('Cq>',  tagInteger + 3, $count);
+    if (havePack64) {
+        return pack('Cq>',  tagInteger + 3, $count);
+    } else {
+        return _pack_int_64(Math::BigInt->new($count)->badd(
+                Math::BigInt->new('0x10000000000000000')));
+    }
 }
 
 sub _pos_integer {
@@ -318,15 +323,24 @@ sub _pos_integer {
 
     if ($count < 256) {
         return pack('CC',  tagInteger + 0, $count);
-    } elsif ($count < 65536) {
+    } elsif ($count <= 65535) {         # 0xFFFF
         return pack('CS>', tagInteger + 1, $count);
-    } elsif (havePack64 && ($count > 4294967295)) {
+    } elsif ($count <= 4294967295) {    # 0xFFFFFFFF
+        return pack('CN',  tagInteger + 2, $count);
+    } elsif (havePack64) {
         return pack('Cq>', tagInteger + 3, $count);
     } else {
-        return pack('CN',  tagInteger + 2, $count);
+        return _pack_int_64($count);
     }
 }
 
+sub _pack_int_64 {
+    my($val) = @_;
+    my $low = Math::BigInt->new($val)->bmod(Math::BigInt->new('0x100000000'));
+    my $high = Math::BigInt->new($val)->brsft(32);
+    return pack( 'CNN', tagInteger + 3, $high->numify, $low->numify);
+}
+
 package Mac::PropertyList::array;
 
 sub _as_bplist_fragment {
@@ -401,9 +415,9 @@ sub _as_bplist_fragment {
     # Therefore all negative numbers must be written as 8-byte
     # integers.
 
-	my $method = $value < 0 ? '_neg_integer' : '_pos_integer';
-	my $sub = Mac::PropertyList::WriteBinary->can($method);
-	$sub->($value);
+    my $method = $value < 0 ? '_neg_integer' : '_pos_integer';
+    my $sub = Mac::PropertyList::WriteBinary->can($method);
+    $sub->($value);
 }
 
 package Mac::PropertyList::uid;
